/*
 * motor_drv.c
 *
 * Created: 2016-04-06 07:10:05
 *  Author: dsosnowski
 */ 




#include <avr/io.h>
#include "defs.h"
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include "wdog.h"
#include "power.h"
#include "ui.h"


//-----------------------------------------------------------------------
void HardInit(void);


#define MAIN_DELAY			15
#define LED_BLINK			10
#define LED_PERIOD			150

#define CHANGE_DELAY		90
#define CHANGE_SHORT_DELAY	10

#define POWER_IDLE_TIME		90	

//-----------------------------------------------------------------------
struct {
	char led;
	char direction;
	char next_dir;
	char change_delay;
	char idle_time;
	char extstop;
} system;





//-----------------------------------------------------------------------
int main(void)
{
	char regval;
	uint16_t overcurrent = 0;
	uint16_t countdown = 0;
	HardInit();
	UiInit();
	
	UiAdcProc();
	UiAdcProc();
	UiAdcProc();
	
	system.change_delay = CHANGE_DELAY;
	
	PowerInit();	
	sei();
    while(1)
    {
		//------------------------------------------------------------
		UiAdcProc();
		UiExtProc();
		
		//------------------------------------------------------------
		regval = UiParamGet(0);		
		if (system.direction == 0){
			if (regval < REG_MIN_TH) {
				if (regval > REG_MIN_H) regval = REG_MIN_H;
				if (regval < REG_MIN_L) regval = REG_MIN_L;
				regval -= REG_MIN_L;
				regval = (PWM_RESOLUTION - regval);
				if (regval < PWM_FILTER) regval = 0;
			} else {
				system.next_dir = 1;			
			}
						
		} else {
			if (regval > REG_MAX_TH) {
				if (regval > REG_MAX_H) regval = REG_MAX_H;
				if (regval < REG_MAX_L) regval = REG_MAX_L;
				regval -= REG_MAX_L;
				if (regval < PWM_FILTER) regval = 0;
			} else {
				system.next_dir = 0;			
			}

		}

		//------------------------------------------------------------
		if (system.direction != system.next_dir) {
			system.direction = system.next_dir;
			if (system.idle_time >= POWER_IDLE_TIME) {
				if (system.change_delay < CHANGE_SHORT_DELAY) { 
					system.change_delay = CHANGE_SHORT_DELAY;
				}
			} else {
				system.change_delay = CHANGE_DELAY;
			}
		}

		//------------------------------------------------------------
		if (UiParamGet(1) < 245) {
			if (PowerGetActual() >= PWM_MAX) {
				overcurrent = UiParamGet(1);
			} else {
				overcurrent = 0xFFFF;
			}
		} else {
			overcurrent = 0xFFFF;
		}


		//------------------------------------------------------------
		if (UiParamGet(2) < 245) {
			if (PowerGetActual() > 0) {
				countdown++;
			} else {
				countdown = 0;	
			}
		} else {
			countdown = 0;
		}

		
		if (system.direction == 0) {
			if (UiParamGet(3) >	overcurrent) system.extstop |= 1;
			if (countdown >= (UiParamGet(2) * 32)) system.extstop |= 1;
			
			if (UiExtGet() & 1) {
				system.extstop |= 1;				
			}
			system.extstop &= ~(2);
		} else {
			if (UiParamGet(3) >	overcurrent) system.extstop |= 2;
			if (countdown >= (UiParamGet(2) * 32)) system.extstop |= 2;

			if (UiExtGet() & 2) {
				system.extstop |= 2;
			}
			system.extstop &= ~(1);
		}
		if (system.extstop) regval = 0;

		//------------------------------------------------------------
		if (system.change_delay < CHANGE_SHORT_DELAY) {
			if (system.direction == 0) REL_OFF;
			else REL_SET;
		}

		//------------------------------------------------------------
		if (regval == 0) {
			if (system.idle_time <  POWER_IDLE_TIME) {
				system.idle_time++;
				} else {
				if (system.change_delay < CHANGE_SHORT_DELAY) {
					system.change_delay = (CHANGE_SHORT_DELAY + 1);
					REL_OFF;
				}
			}
		} else {
			system.idle_time = 0;
		}
		
		//------------------------------------------------------------
		if (system.change_delay) {
			system.change_delay--;
			regval = 0;
		}
		PowerSetValue(regval);
		
		//------------------------------------------------------------		
		system.led++;
		if (regval > 0) {
				LED_SET;			
		} else {
			if (system.led <= 1) LED_SET;
			if (system.led == LED_BLINK) LED_OFF;

			if (system.extstop > 0) {				
				if (system.idle_time < POWER_IDLE_TIME) {
					if (system.led >= (LED_BLINK + LED_BLINK)) {
						system.led = 0;
					}
				}
			}			
			if (system.led >= LED_PERIOD) system.led = 0;
		}
		
		WdtReset();
		_delay_ms(MAIN_DELAY);
	}	
}


//-----------------------------------------------------------------------
void HardInit(void){
	PRR = (1<<PRUSI);
	ACSR |= (ACD);
	WdtSet(7);
}