/*
 * ui.c
 *
 * Created: 2016-04-28 13:59:49
 *  Author: dsosnowski
 */ 

#include <avr/io.h>
#include "defs.h"
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include "ui.h"


//-----------------------------------------------------------------------
#define ANALOG_CHANNEL_REG			(REG_CH)		//vref = vcc, ch3
#define ANALOG_CHANNEL_TRIM1		(TRIM_CURR_CH)	//vref = vcc, ch3
#define ANALOG_CHANNEL_TRIM2		(TRIM_TIME_CH)	//vref = vcc, ch3
//#define ANALOG_CHANNEL_SENSE		(1<<REFS1 | 1<<REFS0 | 0x1F)	//vref = 1,1V, ch 6-7, gain 20x
#define ANALOG_CHANNEL_SENSE		(1<<REFS1 | 1<<REFS0 | SENSE_CH)	//vref = 1,1V, ch 6-7, gain 20x

#define ANALOG_AVERAGE_CYCLES		8
#define ANALOG_CHANNELS				3 


#define EXT_FILTER					3
#define EXT_HYST					10

#define TRIM_MIN					5

struct {
	char vreg;
	char vcurr;
	char vtime;
	char vsense;
	char index;
	char ext;
} analog;


//-----------------------------------------------------------------------
void UiInit(void) {
	
	LED_PORT &= ~(1<<LED_PIN);
	LED_DDR |= (1<<LED_PIN);	
	//EXTREME_PORT |= (1<<EXT1_PIN | 1<<EXT2_PIN);
	
	DIDR0 = (1<<ADC1D | 1<<ADC3D | 1<<ADC4D | 1<<ADC6D | 1<<ADC7D);
	ADCSRA = (1<<ADEN | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);
	ADMUX = (0 | REG_CH);
	ADCSRA |= (1<<ADSC);	
	
	analog.index = ANALOG_CHANNELS;
}

//-----------------------------------------------------------------------
int16_t UiAdcGet(char channel) {
	int16_t buff;
	char i;

	while (ADCSRA & (1<<ADSC));

	ADMUX = channel;
	ADCSRA |= (1<<ADSC);
	while (ADCSRA & (1<<ADSC));

	buff = 0;
	for(i = 0; i < ANALOG_AVERAGE_CYCLES; i++) {
		ADCSRA |= (1<<ADSC);
		while (ADCSRA & (1<<ADSC));
		buff += ADC;
	}
	
	return (buff / ANALOG_AVERAGE_CYCLES);
}

//-----------------------------------------------------------------------
char UiAdcProc(void) {
	
	analog.index++;
	if (analog.index > ANALOG_CHANNELS) analog.index = 0;
	
	if (analog.index == 0) {
		analog.vreg = (UiAdcGet(ANALOG_CHANNEL_REG) / 4);
	} else if (analog.index == 1) {
		analog.vcurr = (UiAdcGet(ANALOG_CHANNEL_TRIM1) / 4);
		analog.vcurr = (0xFF - analog.vcurr);
		if (analog.vcurr < TRIM_MIN) analog.vcurr = TRIM_MIN;
	} else if (analog.index == 2) {
		analog.vtime = (UiAdcGet(ANALOG_CHANNEL_TRIM2) / 4);
		analog.vtime = (0xFF - analog.vtime);
		if (analog.vtime < TRIM_MIN) analog.vtime = TRIM_MIN;
	} else {
		analog.vsense = (UiAdcGet(ANALOG_CHANNEL_SENSE) / 4);
	}
	return analog.index;
}

//-----------------------------------------------------------------------
char UiParamGet (char index) {
	if (index == 0) return analog.vreg;
	if (index == 1) return analog.vcurr;
	if (index == 2) return analog.vtime;
	if (index == 3) return analog.vsense;
	return 0;
}

//-----------------------------------------------------------------------
char UiExtProc(void) {
	static char ext1_counter;
	static char ext2_counter;
	
	if (EXT1_GET == 0){
		if (ext1_counter < EXT_FILTER) ext1_counter++;
		else ext1_counter = EXT_HYST;
	} else {
		if (ext1_counter > EXT_FILTER) ext1_counter--;
		else ext1_counter = 0;
	}
	if (EXT2_GET == 0){
		if (ext2_counter < EXT_FILTER) ext2_counter++;
		else ext2_counter = EXT_HYST;
		} else {
		if (ext2_counter > EXT_FILTER) ext2_counter--;
		else ext2_counter = 0;
	}

	if (ext1_counter > EXT_FILTER) analog.ext |= 1;
	else  analog.ext &= ~(1);
	if (ext2_counter > EXT_FILTER) analog.ext |= 2;
	else  analog.ext &= ~(2);	
	
	return analog.ext;
}

//-----------------------------------------------------------------------
char UiExtGet(void) {
	return analog.ext;
}